import os
import sys
import time
from typing import Callable, Dict, List, Union

import jelka_validator.datawriter as dw

from .shapes import Shape
from .types import AABB
from .types import Color
from .types import Position
from .util import bounds


class Jelka:
    """Main class for controlling the tree."""

    def __init__(
        self,
        frame_rate: int = 60,
        initial_color: Color = Color(0, 0, 0),
        number_of_lights: Union[int, None] = None,
        custom_positions: Union[str, None] = None,
        frame_time_warning: bool = True,
    ):
        self.frame_rate: int = frame_rate
        """Frame rate of the pattern."""

        self.frame: int = 0
        """Current frame number."""

        self.start_time: float = time.perf_counter()
        """Time when the pattern was started."""

        self.elapsed_time: float = 0.0
        """Time since the pattern was started."""

        self.positions_raw: Dict[int, Position] = dict()
        """Raw positions of the lights."""

        self.positions_normalized: Dict[int, Position] = dict()
        """Normalized positions of the lights."""

        self.bounds_raw: AABB = AABB(0, 0, 0, 0, 0, 0)
        """Bounds of the tree in raw coordinates."""

        self.bounds_normalized: AABB = AABB(0, 0, 0, 0, 0, 0)
        """Bounds of the tree in normalized coordinates."""

        self.center_raw: Position = Position(0, 0, 0)
        """Center of the tree in raw coordinates."""

        self.center_normalized: Position = Position(0, 0, 0)
        """Center of the tree in normalized coordinates."""

        if custom_positions:
            # Use a custom positions file if provided
            filenames = [custom_positions]

        elif environment_positions := os.getenv("JELKA_POSITIONS"):
            # Use positions file from environment variable if provided
            filenames = [environment_positions]

        else:
            # Provide default file locations
            filenames = [
                os.path.join(os.getcwd(), "positions.csv"),
                os.path.join(os.path.dirname(sys.argv[0]), "positions.csv"),
                os.path.join(os.getcwd(), "../../data/positions.csv"),
                os.path.join(os.path.dirname(sys.argv[0]), "../../data/positions.csv"),
            ]

        # Resolve relative paths to absolute paths
        filenames = [os.path.abspath(filename) for filename in filenames]

        # Try to load positions from various files
        self.load_positions(filenames)

        # Normalize the positions
        self.normalize_positions(0, 1)

        # Get max index of the lights
        if number_of_lights is None:
            number_of_lights = max(self.positions_raw.keys()) + 1

        self.number_of_lights: int = number_of_lights
        """Number of lights the pattern supports."""

        self.lights: List[Color] = [initial_color for _ in range(number_of_lights)]
        """List of colors of the lights."""

        self.objects: List[Shape] = []
        """List of objects in the scene."""

        self.dw = dw.DataWriter(number_of_lights)
        """Direct access to the data writer."""

        self.clear = False
        """Whether to clear the lights before each frame."""

        self.frame_time_warning = frame_time_warning
        """Whether to show warning if frame time exceeds the limit."""

    def load_positions(self, filenames: List[str]):
        """Loads positions from the first available file."""

        for filename in filenames:
            if not os.path.isfile(filename):
                continue

            with open(filename) as file:
                print(f"[LIBRARY] Loading positions from '{filename}'.", file=sys.stderr, flush=True)

                # Read the positions from the file
                for line in file.readlines():
                    line = line.strip()
                    if line == "":
                        continue
                    i, x, y, z = line.split(",")
                    self.positions_raw[int(i)] = Position(float(x), float(y), float(z))

                # Calculate the bounds of the tree
                self.bounds_raw = bounds(self.positions_raw.values())

                # Calculate the center of the tree
                self.center_raw = Position(0, 0, (self.bounds_raw.min_z + self.bounds_raw.max_z) / 2)

                return

        raise FileNotFoundError(f"[LIBRARY] No valid file found to load positions from (attempted: {filenames}).")

    def normalize_positions(
        self,
        l: int = 0,
        r: int = 1,
        mn: Union[int, float, None] = None,
        mx: Union[int, float, None] = None,
    ):
        """Normalizes the positions to the range [l, r]."""

        if mn is None:
            mn = min([pos.x for pos in self.positions_raw.values()])
            mn = min(min([pos.y for pos in self.positions_raw.values()]), mn)
            mn = min(min([pos.z for pos in self.positions_raw.values()]), mn)

        if mx is None:
            mx = max([pos.x for pos in self.positions_raw.values()])
            mx = max(max([pos.y for pos in self.positions_raw.values()]), mx)
            mx = max(max([pos.z for pos in self.positions_raw.values()]), mx)
            mx += 0.01  # to avoid division by zero

        for i, pos in self.positions_raw.items():
            x = (pos.x - mn) / (mx - mn) * (r - l) + l
            y = (pos.y - mn) / (mx - mn) * (r - l) + l
            z = (pos.z - mn) / (mx - mn) * (r - l) + l
            self.positions_normalized[i] = Position(x, y, z)

        # Calculate the bounds of the tree
        self.bounds_normalized = bounds(self.positions_normalized.values())

        # Calculate the center of the tree
        self.center_normalized = Position(
            (self.center_raw.x - mn) / (mx - mn) * (r - l) + l,
            (self.center_raw.y - mn) / (mx - mn) * (r - l) + l,
            (self.center_raw.z - mn) / (mx - mn) * (r - l) + l,
        )

    def set_light(self, i: int, color: Color):
        """Sets the color of the specified light."""
        self.lights[i] = color

    def run(self, callback: Union[Callable[["Jelka"], None], None] = None, init: Union[Callable[["Jelka"], None], None] = None):
        """Runs the main loop of the tree."""

        # Call the init function
        if init is not None:
            print("[LIBRARY] Running the init function.", file=sys.stderr, flush=True)
            init(self)

        print("[LIBRARY] Starting the main loop.", file=sys.stderr, flush=True)

        self.start_time = time.perf_counter()

        while True:
            self.elapsed_time = time.perf_counter() - self.start_time
            current_time = time.perf_counter_ns()

            # Clear the lights if needed
            if self.clear:
                for light in range(self.number_of_lights):
                    self.set_light(light, Color(0, 0, 0))

            # Update the positions of the objects
            for obj in self.objects:
                obj.update_position(self.elapsed_time)

                for light, position in self.positions_normalized.items():
                    if obj.is_inside(position):
                        self.set_light(light, Color(obj.color.red, obj.color.green, obj.color.blue))

            # Call the callback function
            if callback is not None:
                callback(self)

            # Write the frame to the tree
            writable = [light.to_write() for light in self.lights]
            self.dw.write_frame(writable)

            new_time = time.perf_counter_ns()
            frame_time = new_time - current_time
            target_time = 10**9 / self.frame_rate

            if frame_time <= target_time:
                time.sleep((target_time - frame_time) / 10**9)
            elif self.frame_time_warning:
                print(
                    f"[LIBRARY] Warning: Cannot keep up with the frame rate during the frame {self.frame}.",
                    file=sys.stderr,
                    flush=True,
                )
                print(
                    f"[LIBRARY] Frame time: {frame_time} ns, Max frame time: {target_time} ns",
                    file=sys.stderr,
                    flush=True,
                )

            self.frame += 1



from typing import Union, Tuple

from ..types import Position


class Plane:
    def __init__(
        self,
        center: Union[Position, Tuple[float, float, float]],
        normal: Union[Position, Tuple[float, float, float]],
    ):
        self.center = Position(*center)
        self.normal = Position(*normal)
        self.d = self.center.dot(self.normal)



import math
import random
from colorsys import hsv_to_rgb, rgb_to_hsv

from jelka import Jelka
from jelka.shapes import Plane
from jelka.types import Color

c1: Color
c2: Color


def callback(jelka: Jelka):
    global c1, c2

    plane = Plane(jelka.center_normalized, (math.sin(jelka.frame / 40), 0, math.cos(jelka.frame / 40)))
    threshold = 50

    if jelka.frame % 150 == 0:
        c1 = Color.random().vivid()
        c1hsv = rgb_to_hsv(c1.red / 255.0, c1.green / 255.9, c1.green / 255.0)
        c2hsv = ((c1hsv[0] * 360 + random.randint(100, 200) % 360) / 360.0, c1hsv[1], c1hsv[2])
        conv = hsv_to_rgb(*c2hsv)
        c2 = Color(conv[0] * 255, conv[1] * 255, conv[2] * 255).vivid()

    for light, position in jelka.positions_normalized.items():
        dcrtica = position.dot(plane.normal)

        if plane.d - threshold <= dcrtica <= plane.d:
            jelka.set_light(light, c1)
        else:
            jelka.set_light(light, c2)


def main():
    jelka = Jelka(60)
    jelka.run(callback)


main()



import math

from jelka import Jelka
from jelka.types import Color, Position
from jelka.util import distance

color: Color


def callback(jelka: Jelka):
    global color

    if jelka.frame % 10 == 0:
        color = Color.vivid(Color.random())

    sphere_center = Position(0.25, 0.25, 0.5)
    rad2 = (math.e ** (math.sin(jelka.frame / jelka.frame_rate * 2))) / 3
    rad1 = (math.e ** (math.cos(jelka.frame / jelka.frame_rate * 2))) / 3

    cnt = 0

    for light, position in jelka.positions_normalized.items():
        dist = distance(sphere_center, position)
        if rad1 >= dist >= rad2:
            cnt += 1
            j = dist / rad1
            if jelka.lights[light] == Color(0, 0, 0):
                jelka.set_light(
                    light,
                    Color(j * color.red, j * color.green, j * color.blue),
                )
            else:
                jelka.set_light(
                    light,
                    Color(j * jelka.lights[light].red, j * jelka.lights[light].green, j * jelka.lights[light].blue),
                )
        else:
            jelka.set_light(light, Color(0, 0, 0))

    if cnt == 0:
        jelka.frame += 2


def main():
    jelka = Jelka(60)
    jelka.run(callback)


main()



import math
from typing import List, Tuple, Union

from ..types import Position


class Path:
    """A path is a sequence of positions that the shape should move through."""

    def __init__(self):
        self.positions: List[Position] = []
        self.durations: List[float] = []
        self.loop = True

    def add_position(self, position: Union[Position, Tuple[float, float, float]], time: float):
        """Adds a position to the path."""

        self.positions.append(Position(*position))
        self.durations.append(time)

    def current_position(self, time: float) -> Position:
        """Returns the current position on the path at the given time."""

        total = 0
        for i in range(len(self.durations)):
            total += self.durations[i]

        if self.loop:
            time = time - math.floor(time / total) * total

        for i in range(len(self.durations)):
            if time <= self.durations[i]:
                # Linear interpolation between this and next position component-wise
                t = time / self.durations[i]
                if i == len(self.positions) - 1:
                    return self.positions[i]
                return Position(
                    self.positions[i][0] + t * (self.positions[i + 1][0] - self.positions[i][0]),
                    self.positions[i][1] + t * (self.positions[i + 1][1] - self.positions[i][1]),
                    self.positions[i][2] + t * (self.positions[i + 1][2] - self.positions[i][2]),
                )
            time -= self.durations[i]

        return self.positions[-1]



from abc import ABC, abstractmethod

from ..types import Color, Position


class Shape(ABC):
    """Base class for all shapes."""

    color: Color = Color(0, 0, 0)
    """Color of the shape."""

    @abstractmethod
    def is_inside(self, position: Position) -> bool:
        """Returns whether the given position is inside the shape."""
        raise NotImplementedError

    @abstractmethod
    def update_position(self, time: float):
        """Updates the position of the shape."""
        raise NotImplementedError



from typing import Tuple, Union

from .path import Path
from .shape import Shape
from ..types import Color, Position


class Sphere(Shape):
    def __init__(
        self,
        center: Union[Position, Tuple[float, float, float]],
        radius: float,
        color: Color = Color(255, 192, 203),
    ):
        self.center: Position = Position(*center)
        self.radius: float = radius
        self.color: Color = color

        self.path = Path()
        self.path.add_position(self.center, 0.01)

    def is_inside(self, position: Position) -> bool:
        return (self.center - position).magnitude() <= self.radius

    def update_position(self, time: float):
        self.center = self.path.current_position(time)



from jelka import Jelka
from jelka.types import Color


def callback(jelka: Jelka):
    jelka.set_light(0, color=Color(0, 255, 0))
    jelka.set_light(1, color=Color(0, 255, 0))
    jelka.set_light(2, color=Color(0, 255, 0))
    jelka.set_light(3, color=Color(0, 255, 0))


def main():
    jelka = Jelka(60)
    jelka.run(callback)


main()



from jelka import Jelka
from jelka.types import Color

axis = 0
threshold = 0.05
dimmer = 0.2

color: Color


def wave(x: float, low: float = -0.2, high: float = 1.2) -> float:
    return 2 * (high - low) * abs(x - round(x)) + low


def init(jelka: Jelka):
    mx = max([pos[axis] for pos in jelka.positions_raw.values()])
    mn = min([pos[axis] for pos in jelka.positions_raw.values()])
    jelka.normalize_positions(0, 1, mn, mx)


def callback(jelka: Jelka):
    global color

    if jelka.frame % 300 == 0:
        color = Color.random().vivid()

    coord = wave(jelka.frame / jelka.frame_rate / 4)

    for light, position in jelka.positions_normalized.items():
        jelka.set_light(light, color if abs(position[axis] - coord) < threshold else color * dimmer)


def main():
    jelka = Jelka(60)
    jelka.run(callback, init)


main()



import math
from typing import List

from jelka import Jelka
from jelka.shapes import Sphere
from jelka.types import Color, Position

normalized: List[Position]
color: Color


def init(jelka: Jelka):
    global normalized

    normalized = [Position(0, 0, 0)] * jelka.number_of_lights

    min_x = min([pos.x for pos in jelka.positions_raw.values()])
    max_x = max([pos.x for pos in jelka.positions_raw.values()])
    min_y = min([pos.y for pos in jelka.positions_raw.values()])
    max_y = max([pos.y for pos in jelka.positions_raw.values()])
    min_z = min([pos.z for pos in jelka.positions_raw.values()])
    max_z = max([pos.z for pos in jelka.positions_raw.values()])

    for light, position in jelka.positions_raw.items():
        normalized[light] = Position(
            (position.x - min_x) / (max_x - min_x + 0.01),
            (position.y - min_y) / (max_y - min_y + 0.01),
            (position.z - min_z) / (max_z - min_z + 0.01),
        )


def callback(jelka: Jelka):
    global color

    height = 1 - 0.0075 * (jelka.frame % 150)

    if height == 1:
        color = Color.vivid(Color.random())

    rad = 1 / 2 - height / 2
    x = 0.5 + rad * math.cos(height * 20)
    y = 0.5 + rad * math.sin(height * 20)

    sphere = Sphere((x, y, height), 0.1)

    for i in range(len(jelka.lights)):
        position = normalized[i]
        if sphere.is_inside(position):
            jelka.set_light(i, color)
        # else:
        #   jelka.set_light(i, Color(0, 0, 0))


def main():
    jelka = Jelka(60)
    jelka.run(callback, init)


main()



from dataclasses import dataclass

from .position import Position


@dataclass
class AABB:
    """Represents an Axis-Aligned Bounding Box in 3D space."""

    min_x: float
    """Minimal value on the x-axis."""
    min_y: float
    """Minimal value on the y-axis."""
    min_z: float
    """Minimal value on the z-axis."""
    max_x: float
    """Maximal value on the x-axis."""
    max_y: float
    """Maximal value on the y-axis."""
    max_z: float
    """Maximal value on the z-axis."""

    @property
    def center_x(self) -> float:
        """Center position on the x-axis."""
        return (self.min_x + self.max_x) / 2

    @property
    def center_y(self) -> float:
        """Center position on the y-axis."""
        return (self.min_y + self.max_y) / 2

    @property
    def center_z(self) -> float:
        """Center position on the z-axis."""
        return (self.min_z + self.max_z) / 2

    @property
    def volume(self) -> float:
        """The volume of the AABB."""
        return (self.max_x - self.min_x) * (self.max_y - self.min_y) * (self.max_z - self.min_z)

    def intersects(self, other: "AABB") -> bool:
        """Returns whether this AABB intersects with another AABB."""
        return (
            self.min_x <= other.max_x
            and self.max_x >= other.min_x
            and self.min_y <= other.max_y
            and self.max_y >= other.min_y
            and self.min_z <= other.max_z
            and self.max_z >= other.min_z
        )

    def contains(self, position: "Position") -> bool:
        """Returns whether this AABB contains a specific position."""
        return (
            self.min_x <= position.x <= self.max_x
            and self.min_y <= position.y <= self.max_y
            and self.min_z <= position.z <= self.max_z
        )



import random
from typing import List, Tuple, Union


class Color(Tuple[float, float, float]):
    """Represents an RGB color."""

    def __new__(cls, red: float, green: float, blue: float):
        return super().__new__(cls, (red, green, blue))

    @property
    def red(self) -> float:
        return self[0]

    @property
    def green(self) -> float:
        return self[1]

    @property
    def blue(self) -> float:
        return self[2]

    def __add__(self, other: "Color") -> "Color":  # pyright: ignore[reportIncompatibleMethodOverride]
        return Color(self.red + other.red, self.green + other.green, self.blue + other.blue)

    def __sub__(self, other: "Color") -> "Color":  # pyright: ignore[reportIncompatibleMethodOverride]
        return Color(self.red - other.red, self.green - other.green, self.blue - other.blue)

    def __mul__(self, other: Union["Color", int, float]) -> "Color":  # pyright: ignore[reportIncompatibleMethodOverride]
        if isinstance(other, Color):
            return Color(self.red * other.red, self.green * other.green, self.blue * other.blue)
        elif isinstance(other, (int, float)):
            return Color(self.red * other, self.green * other, self.blue * other)

        raise TypeError("Unsupported operand type(s) for *: 'Color' and '{}'".format(type(other).__name__))

    def __truediv__(self, other: Union["Color", int, float]) -> "Color":  # pyright: ignore[reportIncompatibleMethodOverride]
        if isinstance(other, Color):
            return Color(self.red / other.red, self.green / other.green, self.blue / other.blue)
        elif isinstance(other, (int, float)):
            return Color(self.red / other, self.green / other, self.blue / other)

        raise TypeError("Unsupported operand type(s) for /: 'Color' and '{}'".format(type(other).__name__))

    def __eq__(self, other) -> bool:
        if not isinstance(other, Color):
            return False

        return self.red == other.red and self.green == other.green and self.blue == other.blue

    def __str__(self) -> str:
        return f"Color({self.red}, {self.green}, {self.blue})"

    def __repr__(self) -> str:
        return f"Color({self.red}, {self.green}, {self.blue})"

    def to_tuple(self) -> Tuple[float, float, float]:
        """Returns the color as a tuple of floats."""
        return self.red, self.green, self.blue

    def to_list(self) -> List[float]:
        """Returns the color as a list of floats."""
        return [self.red, self.green, self.blue]

    def to_write(self) -> Tuple[int, int, int]:
        """Returns the color in a format for the data writer."""

        def round_clamp(value):
            return max(0, min(255, round(value)))

        return round_clamp(self.red), round_clamp(self.green), round_clamp(self.blue)

    def vivid(self):
        """Returns a vivid version of the color."""

        minimal = min(self.red, self.green, self.blue)

        # Remove the smallest component to make the color more vibrant
        if self.red == minimal:
            return Color(0, self.green, self.blue)
        elif self.green == minimal:
            return Color(self.red, 0, self.blue)
        elif self.blue == minimal:
            return Color(self.red, self.green, 0)

        return self

    @staticmethod
    def random() -> "Color":
        """Returns a random color."""
        return Color(random.uniform(0, 255), random.uniform(0, 255), random.uniform(0, 255))



import math

from jelka import Jelka
from jelka.shapes import Sphere
from jelka.types import Color


def callback(jelka: Jelka):
    cols = [Color(255, 0, 0), Color(0, 0, 255), Color(255, 255, 255)]
    height = [1 - 0.005 * (jelka.frame % 210), 1 - 0.005 * ((jelka.frame + 70) % 210), 1 - 0.005 * ((jelka.frame + 140) % 210)]

    rad = [(1 - height[0]) / 2, (1 - height[1]) / 2, (1 - height[2]) / 2]

    x = [
        0.5 + rad[0] * math.cos(height[0] * 20),
        0.5 + rad[1] * math.cos(height[1] * 20),
        0.5 + rad[2] * math.cos(height[2] * 20),
    ]
    y = [
        0.5 + rad[0] * math.sin(height[0] * 20),
        0.5 + rad[1] * math.sin(height[1] * 20),
        0.5 + rad[2] * math.sin(height[2] * 20),
    ]

    sph = [
        Sphere((x[0], y[0], height[0]), 0.2),
        Sphere((x[1], y[1], height[1]), 0.2),
        Sphere((x[2], y[2], height[2]), 0.2),
    ]

    for light, position in jelka.positions_normalized.items():
        for j in range(0, 3):
            if sph[j].is_inside(position):
                jelka.set_light(light, cols[j])


def main():
    jelka = Jelka(60)
    jelka.run(callback)


main()



from simulation import Simulation

from subprocess import Popen, PIPE
import sys
import time
from jelka_validator.datareader import DataReader


if __name__ == "__main__":
    # Popen(["-m", "writer.py"], executable=sys.executable, stdout=PIPE)
    # Popen(["writer.exe"], stdout=PIPE)
    smreka = {}
    with open("positions.csv") as f:
        for line in f.readlines():
            line = line.strip()
            if line == "":
                continue
            i, x, y, z = line.split(",")
            smreka[int(i)] = (float(x), float(y), float(z))

    with Popen([sys.executable, "rotating_full.py"], stdout=PIPE) as p:
        sim = Simulation(smreka)
        dr = DataReader(p.stdout.read1)  # type: ignore
        dr.update()
        # assert dr.header is not None
        sim.init()
        time.sleep(1)
        while sim.running:
            c = next(dr)
            # assert all(c[i] == c[0] for i in range(len(c)))
            dr.user_print()
            sim.set_colors(dict(zip(range(len(c)), c)))
            sim.frame()
        sim.quit()



from typing import Tuple


class Position(Tuple[float, float, float]):
    """Represents a position in 3D space."""

    def __new__(cls, x: float, y: float, z: float):
        return super().__new__(cls, (x, y, z))

    @property
    def x(self) -> float:
        return self[0]

    @property
    def y(self) -> float:
        return self[1]

    @property
    def z(self) -> float:
        return self[2]

    def __add__(self, other: "Position") -> "Position":  # pyright: ignore[reportIncompatibleMethodOverride]
        return Position(self.x + other.x, self.y + other.y, self.z + other.z)

    def __sub__(self, other: "Position") -> "Position":  # pyright: ignore[reportIncompatibleMethodOverride]
        return Position(self.x - other.x, self.y - other.y, self.z - other.z)

    def __mul__(self, scalar: float) -> "Position":  # pyright: ignore[reportIncompatibleMethodOverride]
        return Position(self.x * scalar, self.y * scalar, self.z * scalar)

    def __truediv__(self, scalar: float) -> "Position":  # pyright: ignore[reportIncompatibleMethodOverride]
        return Position(self.x / scalar, self.y / scalar, self.z / scalar)

    def __eq__(self, other) -> bool:
        if not isinstance(other, Position):
            return False

        return self.x == other.x and self.y == other.y and self.z == other.z

    def __str__(self):
        return f"Position({self.x}, {self.y}, {self.z})"

    def __repr__(self):
        return f"Position({self.x}, {self.y}, {self.z})"

    def dot(self, other: "Position") -> float:
        """Calculates the dot product of two positions."""
        return self.x * other.x + self.y * other.y + self.z * other.z

    def cross(self, other: "Position") -> "Position":
        """Calculates the cross product of two positions."""
        return Position(
            self.y * other.z - self.z * other.y,
            self.z * other.x - self.x * other.z,
            self.x * other.y - self.y * other.x,
        )

    def magnitude(self) -> float:
        """Calculates the magnitude of the position."""
        return (self.x**2 + self.y**2 + self.z**2) ** 0.5

    def normalize(self) -> "Position":
        """Returns the normalized position."""
        mag = self.magnitude()
        return self / mag if mag != 0 else self



import random as r

from jelka import Jelka
from jelka.shapes import Sphere
from jelka.types import Color


def init(jelka: Jelka):
    jelka.clear = True

    for i in range(0, 10):
        sphere = Sphere(
            (0.5 + r.uniform(-0.2, 0.2), 0.5 + r.uniform(-0.2, 0.2), 1 + r.uniform(0.0, 1.0)),
            0.1,
            Color(2, 0, 121),
        )

        sphere.path.add_position(
            sphere.center,
            3 + r.uniform(-1.0, 1.0),
        )

        sphere.path.add_position(
            (r.uniform(-1.0, 1.0), r.uniform(-1.0, 1.0), -0.5 + r.uniform(-2.0, 0.4)),
            1 + r.uniform(-0.2, 0.2),
        )

        jelka.objects.append(sphere)


def callback(jelka: Jelka):
    return None


def main():
    jelka = Jelka(60)
    jelka.run(callback, init)


main()



from colorsys import hsv_to_rgb, rgb_to_hsv

from jelka import Jelka
from jelka.types import Color


def callback(jelka: Jelka):
    for i in range(jelka.number_of_lights):
        hue = rgb_to_hsv(jelka.lights[i].red, jelka.lights[i].green, jelka.lights[i].blue)[0] * 360
        hue = (hue + 4) % 360

        color = hsv_to_rgb(hue / 360.0, 1.0, 1.0)
        color = tuple(map(lambda x: int(x * 255), color))
        jelka.set_light(i, Color(color[0], color[1], color[2]))


def main():
    jelka = Jelka(60)
    jelka.run(callback)


main()



import math

from jelka import Jelka
from jelka.types import Color


def callback(jelka: Jelka):
    for light, position in jelka.positions_normalized.items():
        jelka.set_light(
            light,
            Color(
                (position[0] * 255 + math.sin(jelka.elapsed_time + 1) * 255 + 256) % 256,
                (position[1] * 255 + math.sin(jelka.elapsed_time + 2) * 255 + 256) % 256,
                (position[2] * 255 + math.sin(jelka.elapsed_time) * 255 + 256) % 256,
            ).vivid(),
        )


def main():
    jelka = Jelka(60)
    jelka.run(callback)


main()



from jelka import Jelka
from jelka.types import Color


color: Color = Color.random().vivid()
spawn_rate: int = 40
line_length: int = 10


def callback(jelka: Jelka):
    global color
    global spawn_rate
    global line_length

    if (jelka.frame % spawn_rate) < line_length:
        jelka.set_light(0, color=color)
    else:
        jelka.set_light(0, color=Color(0, 0, 0))
        color = Color.random().vivid()

    for i in reversed(range(1, jelka.number_of_lights)):
        jelka.set_light(i, jelka.lights[i - 1])


def main():
    jelka = Jelka(40)
    jelka.run(callback)


main()



from typing import Iterable, Tuple, Union

from .types import AABB, Position


def length(position: Union[Position, Tuple[float, float, float]]) -> float:
    """Calculates an absolute value of a point."""
    return Position(*position).magnitude()


def distance(p1: Union[Position, Tuple[float, float, float]], p2: Union[Position, Tuple[float, float, float]]) -> float:
    """Calculates a distance between two points."""
    return (Position(*p1) - Position(*p2)).magnitude()


def bounds(positions: Iterable[Union[Position, Tuple[float, float, float]]]) -> AABB:
    """Calculates a bounding box of positions."""
    min_x = min(pos[0] for pos in positions)
    min_y = min(pos[1] for pos in positions)
    min_z = min(pos[2] for pos in positions)
    max_x = max(pos[0] for pos in positions)
    max_y = max(pos[1] for pos in positions)
    max_z = max(pos[2] for pos in positions)

    return AABB(min_x, min_y, min_z, max_x, max_y, max_z)
